/*! autodux v4.0.2 by Eric Elliott */
'use strict';

require('core-js/modules/es6.array.reduce-right');
require('core-js/modules/es6.regexp.split');
require('core-js/modules/es6.object.assign');
require('core-js/modules/web.dom.iterable');
require('core-js/modules/es6.array.iterator');
require('core-js/modules/es6.object.keys');
require('core-js/modules/es6.array.reduce');

var get = require('lodash/fp/get');

var capitalize = require('lodash/upperFirst');

var id = function id(x) {
  return x;
};

var selectIf = function selectIf(predicate) {
  return function (x) {
    return predicate(x) && x;
  };
};

var isFunction = function isFunction(f) {
  return typeof f == 'function';
};

var selectFunction = selectIf(isFunction);

var toGetter = function toGetter(s) {
  return "get" + capitalize(s);
};

var sliceSelector = function sliceSelector(slice, fn) {
  return function (state) {
    return fn(state[slice], state);
  };
};

var createInitSelectors = function createInitSelectors(slice, initial) {
  return Object.keys(initial).reduce(function (obj, selector) {
    var _Object$assign, _Object$assign2;

    return slice ? Object.assign(obj, (_Object$assign = {}, _Object$assign[toGetter(selector)] = sliceSelector(slice, function (state) {
      return get(selector, state);
    }), _Object$assign)) : Object.assign(obj, (_Object$assign2 = {}, _Object$assign2[toGetter(selector)] = function (state) {
      return get(selector, state);
    }, _Object$assign2));
  }, {});
};

var createSliceSelectors = function createSliceSelectors(slice, selectors) {
  var _Object$assign5;

  return Object.assign(Object.keys(selectors).reduce(function (obj, selector) {
    var _Object$assign3, _Object$assign4;

    return slice ? Object.assign(obj, (_Object$assign3 = {}, _Object$assign3[selector] = sliceSelector(slice, selectors[selector]), _Object$assign3)) : Object.assign(obj, (_Object$assign4 = {}, _Object$assign4[selector] = selectors[selector], _Object$assign4));
  }, {}), (_Object$assign5 = {}, _Object$assign5[toGetter(slice)] = sliceSelector(slice, id), _Object$assign5));
};

var getActionName = function getActionName(key) {
  return "set" + capitalize(key);
};

var createAction = function createAction(slice, action, key, type) {
  if (type === void 0) {
    type = slice + "/" + action;
  }

  return {
    create: Object.assign(function (payload) {
      return payload;
    }, {
      type: type
    }),
    reducer: function reducer(state, payload) {
      var _ref;

      return Object.assign({}, state, key ? (_ref = {}, _ref[key] = payload, _ref) : payload);
    }
  };
};

var getInitialActions = function getInitialActions(slice, initial) {
  return Object.keys(initial).reduce(function (o, key) {
    var _Object$assign6;

    var action = getActionName(key);
    return Object.assign(o, (_Object$assign6 = {}, _Object$assign6[action] = createAction(slice, action, key), _Object$assign6));
  }, {});
};

var addDefaultActions = function addDefaultActions(slice, initial, actions) {
  var _Object$assign7;

  var initialActions = getInitialActions(slice, initial);
  var action = getActionName(slice);
  return Object.assign({}, (_Object$assign7 = {}, _Object$assign7[action] = createAction(slice, action), _Object$assign7), initialActions, actions);
};

var createMappedActions = function createMappedActions(slice, actions) {
  return Object.keys(actions).reduce(function (obj, action) {
    var _Object$assign8;

    return Object.assign({}, obj, (_Object$assign8 = {}, _Object$assign8[action] = Object.assign(function () {
      var _actions$action;

      return {
        type: slice + "/" + action,
        payload: typeof actions[action].create == 'function' ? (_actions$action = actions[action]).create.apply(_actions$action, arguments) : arguments.length <= 0 ? undefined : arguments[0]
      };
    }, {
      type: slice + "/" + action
    }), _Object$assign8));
  }, {});
};

var autodux = function autodux(_temp) {
  var _ref2 = _temp === void 0 ? {} : _temp,
      _ref2$initial = _ref2.initial,
      initial = _ref2$initial === void 0 ? '' : _ref2$initial,
      _ref2$actions = _ref2.actions,
      actions = _ref2$actions === void 0 ? {} : _ref2$actions,
      _ref2$selectors = _ref2.selectors,
      selectors = _ref2$selectors === void 0 ? {} : _ref2$selectors,
      _ref2$slice = _ref2.slice,
      slice = _ref2$slice === void 0 ? '' : _ref2$slice;

  var allSelectors = Object.assign({}, createInitSelectors(slice, initial), createSliceSelectors(slice, selectors));
  var allActions = createMappedActions(slice, addDefaultActions(slice, initial, actions));
  return {
    initial: initial,
    reducer: function reducer(state, _temp2) {
      if (state === void 0) {
        state = initial;
      }

      var _ref3 = _temp2 === void 0 ? {} : _temp2,
          type = _ref3.type,
          payload = _ref3.payload;

      var _ref4 = type ? type.split('/') : 'unknown/unknown'.split('/'),
          namespace = _ref4[0],
          subType = _ref4[1];

      var defaultActions = addDefaultActions(slice, initial, {});
      var actionReducer = [get(subType + ".reducer", actions), actions[subType], get(subType + ".reducer", defaultActions)].reduceRight(function (f, v) {
        return selectFunction(v) || f;
      });
      return namespace === slice && (actions[subType] || defaultActions[subType]) ? actionReducer ? actionReducer(state, payload) : Object.assign({}, state, payload) : state;
    },
    slice: slice,
    selectors: allSelectors,
    actions: allActions
  };
};

module.exports = autodux;
module.exports.id = id;

module.exports.assign = function (key) {
  return function (state, payload) {
    var _Object$assign9;

    return Object.assign({}, state, (_Object$assign9 = {}, _Object$assign9[key] = payload, _Object$assign9));
  };
};
//# sourceMappingURL=index.js.map
